<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * A class for managing fields of an Active Record.
 * 
 * This class is instantiated by ActiveRecord for every field (or column) found 
 * in the record.
 * 
 * If a comment for the column contains the string 
 * <pre>alchitect(validate:rule)</pre> or <pre>alchitect(sanitize:rule)</pre>
 * then the field will automatically be sanitized or validated based on the 
 * given rule before the data is inserted or updated in the database.
 *
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASActiveRecordField
{
	/**
	 * Whether the field is a foreign key
	 * 
	 * @var Boolean
	 */
	private $foreign_key;
	
	/**
	 * The name of the field.
	 *
	 * @var String
	 */
	private $name;

	/**
	 * Whether the field is the primary key.
	 *
	 * @var Boolean
	 */
	private $primary = false;

	/**
	 * The sanitization rule for this field.
	 * 
	 * @var String
	 */
	private $sanitization_rule;
	
	/**
	 * The field size.
	 *
	 * @var Integer
	 */
	private $size;
	
	/**
	 * Whether the field has been touched, meaning the field has been given
	 * a value.
	 *
	 * @var Boolean
	 */
	private $touched = false;	
	
	/**
	 * The field type.
	 *
	 * @var String
	 */
	private $type;
			
	/**
	 * The validation rule for this field.
	 * 
	 * @var String
	 */
	private $validation_rule;
	
	/**
	 * The value of the field.
	 *
	 * @var Mixed
	 */
	private $value;

	/**
	 * Return whether the current attribute is a foreign key to another table.
	 *
	 * @return Boolean
	 */
	public function isForeignKey()
	{
		return $this->foreign_key;
	}
	
	/**
	 * Return whether the current attribute is the primary key.
	 *
	 * @return Boolean
	 */
	public function isPrimary()
	{
		return $this->primary;
	}
 
	/**
	 * Whether the given field has been touched (given a value).
	 *
	 * @return Boolean
	 */
	public function isTouched()
	{
		return $this->touched;
	}
	
	/**
	 * Get the name of the current attribute.
	 *
	 * @return String
	 */
	public function name()
	{
		return $this->name;
	}

	/**
	 * Get the validation rule if any.
	 *
	 * @return Mixed
	 */
	public function sanitizationRule()
	{
		return $this->sanitization_rule;
	}
	
	/**
	 * Set whether the current attribute is the primary key.
	 *
	 */
	public function setForeignKey()
	{
		$this->foreign_key = true;
	}
	
	/**
	 * Set the internal value of the current attribute.
	 *
	 * @param String $value
	 */
	public function setInternalValue($value)
	{
		$this->value = $value;
	}

	/**
	 * Set the name of the current attribute.
	 *
	 * @param String $name
	 */
	public function setName($name)
	{
		$this->name = $name;
	}
	
	/**
	 * Set whether the current attribute is the primary key.
	 *
	 */
	public function setPrimary()
	{
		$this->primary = true;
	}
	
	/**
	 * Set the size of the current attribute.
	 *
	 * @param Integer $size
	 */
	public function setSize($size)
	{
		$this->size = $size;
	}
	
	/**
	 * Set a sanitization rule for this field.  Valid rules are:
	 * 
	 * <ul>
	 * 	<li><pre>string</pre></li>
	 * 	<li><pre>stripped</pre></li>
	 * 	<li><pre>encoded</pre></li>
	 * 	<li><pre>special_chars</pre></li>
	 * 	<li><pre>email</pre></li>
	 * 	<li><pre>url</pre></li>
	 * 	<li><pre>number_int</pre></li>
	 * 	<li><pre>number_float</pre></li>
	 * 	<li><pre>magic_quotes</pre></li>
	 * </ul>
	 * 
	 * See {@link http://www.php.net/manual/en/filter.constants.php} for more
	 * information on what each rule sanitizes.
	 *
	 * @param String The sanitization rule.
	 */
	public function setSanitizationRule($sanitization_rule)
	{
		$supported_rules = array
		(
			'string'
		  , 'stripped'
		  , 'encoded'
		  , 'special_chars'
		  , 'email'
		  , 'url'
		  , 'number_int'
		  , 'number_float'
		  , 'magic_quotes'
		);
		
		if (in_array($sanitization_rule, $supported_rules))
		{
			$this->sanitization_rule = $sanitization_rule;
		}
	}
	
	/**
	 * Set the type of the current attribute.
	 *
	 * @param String $type
	 */
	public function setType($type)
	{
		$this->type = $type;
	}
		
	/**
	 * Set a validation rule for this field.  Valid rules are:
	 * 
	 * <ul>
	 * 	<li><pre>int</pre></li>
	 * 	<li><pre>boolean</pre></li>
	 * 	<li><pre>float</pre></li>
	 * 	<li><pre>url</pre></li>
	 * 	<li><pre>email</pre></li>
	 * 	<li><pre>ip</pre></li>
	 * </ul>
	 * 
	 * See {@link http://www.php.net/manual/en/filter.constants.php} for more
	 * information on what each rule validates.
	 *
	 * @param String The validation rule.
	 */
	public function setValidationRule($validation_rule)
	{
		$supported_rules = array
		(
			'int'
		  , 'boolean'
		  , 'float'
		  , 'url'
		  , 'email'
		  , 'ip'
		);
		
		if (in_array($validation_rule, $supported_rules))
		{
			$this->validation_rule = $validation_rule;
		}
	}
		
	/**
	 * Set the value of the current attribute.
	 *
	 * @param Mixed $value
	 */
	public function setValue($value)
	{
		$this->value   = $value;
		$this->touched = true;
	}
	
	/**
	 * Get the size of the current attribute.
	 *
	 * @return Integer
	 */
	public function size()
	{
		return $this->size;
	}

	/**
	 * Get the type of the current attribute.
	 *
	 * @return String
	 */
	public function type()
	{
		return $this->type;
	}
	
	/**
	 * Get the validation rule if any.
	 *
	 * @return Mixed
	 */
	public function validationRule()
	{
		return $this->validation_rule;
	}
	
	
	/**
	 * Get the value of the current attribute.
	 *
	 * @return Mixed
	 */
	public function value()
	{
		return $this->value;
	}
}
?>