<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * ASConfig
 * 
 * Class for handling application configuration data.
 * 
 * The class reads in 'Alchitect/config.ini' and stores a serialized version
 * in '/tmp/ASConfig.serialized'.  If the latter file already exists then this
 * class reads and unserializes it instead.
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASConfig
{
	/**
	 * An array of configuration data.
	 * 
	 * @var Array
	 */
	private $config_data = null;
	
	/**
	 * The name of the file in which to store the serialized config data.
	 * 
	 * @var String
	 */
	private static $serialized_config_file = 'tmp/ASConfig.serialized';
	
	/**
	 * The name of the config file.
	 * 
	 * @var String
	 */
	private $config_file = 'config.ini';
	
	/**
	 * An instance of this class.
	 * 
	 * @var ASConfig
	 */
	private static $instance = null;
	
	/**
	 * The constructor
	 */
	private function __construct()
	{
		$this->init();
	}

	public function addAddOnFromXML($add_on_xml)
	{
		$xml	= simplexml_load_string($add_on_xml);
		$add_on	= array
		(
			'name'				=> (string)$xml->name
		  , 'short_name'		=> (string)$xml->short_name
		  , 'version'			=> (string)$xml->version
		  , 'author'			=> array
			(               	
				'name'			=> (string)$xml->author->name
			  , 'email'			=> (string)$xml->author->email
			)               	
		  , 'copyright'			=> (string)$xml->copyright
		  , 'web_site'			=> (string)$xml->web_site
		  , 'license'			=> array
			(               	
				'name'			=> (string)$xml->license->name
			  , 'short_name'	=> (string)$xml->license->short_name
			  , 'web_site'		=> (string)$xml->license->web_site
			)
		  , 'dependencies'		=> array
			(
				'core_server'	=> (string)$xml->dependencies->core_server
			  , 'core_ui'		=> (string)$xml->dependencies->core_ui
			  , 'add_on'		=> (string)$xml->dependencies->add_on
			)
		);

		$this->config_data['add_ons'][] = $add_on;
	}
	
	/**
	 * Get config data for a given item and sub-item.  Otherwise get the entire
	 * {@link $config_data} array.
	 * 
	 * @param String The name of the item to get.
	 * @param String The name of the sub-item to get.
	 * @return Mixed 
	 */
	public function configData($item=null, $sub_item=null)
	{
		if (is_null($item))
		{
			return $this->config_data;
		}
		else
		{
			if (is_null($sub_item))
			{
				return $this->config_data[$item];
			}
			else
			{
				return $this->config_data[$item][$sub_item];
			}
		}
	}
	
	/**
	 * Get the name of the config file.
	 * 
	 * @return String
	 */
	public function configFile()
	{
		return ALCHITECT_BASE_ROOT . '/' . $this->config_file;
	}

	/**
	 * Get a sub-item of the database section.
	 * 
	 * A convenience function for getting database-related config options.
	 * 
	 * @param String The sub-item of the database section.
	 * @return String
	 */
	public function database($sub_item=null)
	{
		return $this->configData('database', $sub_item);
	}
	
	/**
	 * Get the database host name.
	 * 
	 * @return String
	 */
	public function databaseHost()
	{
		return $this->database('host');
	}
	
	/**
	 * Get the database type.
	 * 
	 * @return String
	 */
	public function databaseType()
	{
		return $this->database('type');
	}
		
	/**
	 * Get the database name.
	 * 
	 * @return String
	 */
	public function databaseName()
	{
		return $this->database('name');
	}
	
	/**
	 * Get the database user.
	 * 
	 * @return String
	 */
	public function databaseUser()
	{
		return $this->database('user');
	}
	
	/**
	 * Get the database password.
	 * 
	 * @return String
	 */
	public function databasePassword()
	{
		return $this->database('password');
	}
	
	/**
	 *
	 */
	private function findAddOns()
	{
		$add_ons_dir = dir(ALCHITECT_BASE_ROOT . '/add_ons');
		
		while (false !== ($entry = $add_ons_dir->read()))
		{
			if (false == preg_match('/\.xml$/', $entry))
			{
				continue;
			}
			
			// echo "{$entry}<br />\n";
			$this->addAddOnFromXML(file_get_contents(ALCHITECT_BASE_ROOT . '/add_ons/' . $entry));
		}
		
		$add_ons_dir->close();
	}
	
	/**
	 *  Initialize the $config_data property. 
	 */
	private function init()
	{		
		if (false == is_file($this->configFile()))
		{
			throw new ASException("Unable to find config file: '{$this->configFile()}'");
		}
		
		$this->config_data = parse_ini_file(ALCHITECT_BASE_ROOT . '/config.ini', true);
		
		// $this->findAddOns();
	}
	
	/**
	 * Get an instance of this class.
	 * 
	 * @return ASConfig
	 */
	public static function instance()
	{
		if (is_null(self::$instance))
		{
			if (file_exists(self::$serialized_config_file))
			{
				self::$instance =
					unserialize(file_get_contents
					(
						self::$serialized_config_file
					));
			}
			
			if (is_null(self::$instance))
			{
				self::$instance = new ASConfig();
			}
			
			if (false == is_dir(ALCHITECT_BASE_ROOT . '/tmp'))
			{
				mkdir(ALCHITECT_BASE_ROOT . '/tmp');
			}
			
			if (false == file_exists(ALCHITECT_BASE_ROOT . '/' . self::$serialized_config_file))
			{
				if (false === file_put_contents(self::$serialized_config_file, serialize(self::$instance)))
				{
					throw new ASException(
						"No permission to save configuration file to '" . self::$serialized_config_file . "'"
					);
				}
			}
		}
		
		return self::$instance;
	}
	
	/**
	 * Get a sub-item of the log section.
	 * 
	 * A convenience function for getting log-related config options.
	 * 
	 * @param String The sub-item of the log section.
	 * @return String
	 */
	public function log($sub_item=null)
	{
		return $this->configData('log', $sub_item);
	}
	
	/**
	 * Get whether logging is enabled.
	 * 
	 * @return String
	 */
	public function logEnabled()
	{
		return $this->log('enabled');
	}
	
	/**
	 * Get the log type.
	 * 
	 * @return String
	 */
	public function logType()
	{
		return $this->log('type');
	}
	
	/**
	 * Get the log file path.
	 * 
	 * @return String
	 */
	public function logFilePath()
	{
		return $this->log('file_path');
	}
		
	/**
	 * Get the log level.
	 * 
	 * @return String
	 */
	public function logLevel()
	{
		return $this->log('level');
	}

	/**
	 *
	 */
	public function addOns()
	{
		return $this->configData('add_ons');
	}
	
	/**
	 * Get a sub-item of the pages section.
	 * 
	 * A convenience function for getting pages-related config options.
	 * 
	 * @param String The sub-item of the pages section.
	 * @return String
	 */
	public function pages($sub_item=null)
	{
		return $this->configData('pages', $sub_item);
	}
	
	/**
	 * Get the default page.
	 * 
	 * @return String
	 */
	public function pagesDefault()
	{
		return $this->pages('default');
	}

	/**
	 * Get the pages directory.
	 * 
	 * @return String
	 */
	public function pagesDirectory()
	{
		return $this->pages('directory');
	}
	
	/**
	 * Get the pages template file name.
	 * 
	 * @return String
	 */
	public function pagesTemplate()
	{
		return $this->pages('template');
	}
}
?>
