<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * Alchitect base class for the Model/View/Controller pattern.
 * 
 * This class handles various GET, POST, and JSON requests.
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASMVC extends ASApplication implements ASObservable
{
	private $PageRequest = null;
	
	/**
	 * The Constructor
	 *
	 * @access public
	 * @param ASPageRequest The page request object.
	 * @return Void
	 */
	public function __construct(ASPageRequest $PageRequest)
	{
		parent::__construct();
		
		$this->PageRequest = $PageRequest;

		$PageRequest->addObserver($this);
	}
	
	/**
	 * Generic accessor
	 *
	 * @access private
	 * @param String The name of the property to get.
	 * @return Mixed
	 */
	public function __get($var_name)
	{
		if (isset($this->$var_name))
		{
			return $this->$var_name;
		}
		
		return null;
	}

	/**
	 * Generic query method.
	 * 
	 * This is a helper method for executing various db queries based on a given
	 * type.
	 *
	 * @access private 
	 * @param String The type of query to execute. Should be one of 'insert', 
	 * 'update', 'delete'.
	 * @param Array The data to be used in the query.
	 * @return Boolean
	 */
	public function genericQuery($type, $info_data)
	{		
		if (is_array($info_data) && sizeof($info_data))
		{
			foreach ($info_data as $key=>$val)
			{
				$this->Model->setAttrib($key, $val);
			}

			if ('insert' == $type)
			{
				$this->Model->setAttrib('id', ASString::uuid());
				$this->Model->setAttrib('created_on', date('Y-m-d H:i:s'));
				$this->Model->setIgnoredAttrib('last_modified_on');
			}

			if ('update' == $type)
			{
				$this->Model->setIgnoredAttrib('created_on');
				$this->Model->setAttrib
				(
					'last_modified_on'
				  , date('Y-m-d H:i:s')
				);
			}

			try
			{
				return $this->Model->$type();
			}
			catch (Exception $ex)
			{
				throw new ASException
				(
					"Unable to {$type} data - {$ex->getMessage()}"
				  , $this->Model->handle()->errorCode()
				);
			}
		}
		
		return null;
	}

	/**
	 * Get all the data from the Model and return it as an associative array.
	 * 
	 * @return Array
	 */
	public function getAllDataAsArray()
	{
		$all = $this->Model->getAll();
		$data = array();
		
		foreach ($all as $entry)
		{
			$data[] = $entry->getData();
		}
		
		return $data;
	}
	
	/**
	 * Get the data from the model using the ID from ASPageRequest in the 
	 * 'WHERE' clause and return it as an array.  If no data is found then
	 * return an empty array.
	 * 
	 * @return Array
	 */
	public function getDataAsArray()
	{
		$this->Model->setAttrib('id', $this->PageRequest->id());
		
		if ($this->Model->get())
		{
			return $this->Model->getData();
		}
		
		return array();
	}
	
	/**
	 * Gandle HTTP GET requests
	 * 
	 * This method is used for fetching data from the database based on
	 * the ID in the HTTP query string.
	 *
	 * @access public
	 * @see setPropertiesForKeys()
	 */
	public function handleGet()
	{
		if ($this->PageRequest->id())
		{
			$this->Model->setAttrib('id', $this->PageRequest->id());
			$this->Model->get();

			$all_data = $this->Model->getData();

			$this->setPropertiesForKeys(
				array_keys($all_data)
			  , $all_data
			  , $this->Model
			);
		}
	}

	/**
	 * Handle HTTP GET requests with an 'insert' action
	 *
	 * @access public
	 * @param Array The data to be inserted.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleGetInsert($insert_data)
	{
		return $this->genericQuery('insert', $insert_data);
	}

	/**
	 * Handle HTTP GET requests with an 'update' action
	 *
	 * @access public
	 * @param Array The data to be updated.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleGetUpdate($update_data)
	{
		return $this->genericQuery('update', $update_data);
	}

	/**
	 * Handle HTTP GET requests with a 'delete' action
	 *
	 * @access public
	 * @param Array The data to be deleted.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleGetDelete($delete_data)
	{
		return $this->genericQuery('delete', $delete_data);
	}

	/**
	 * Handle JSON requests with an 'delete' action
	 *
	 * @access public
	 * @param Array The data to be deleted.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleJSONDelete($delete_data)
	{
		return $this->genericQuery('delete', $delete_data);
	}
	
	/**
	 * Handle JSON requests with an 'insert' action
	 *
	 * @access public
	 * @param Array The data to be inserted
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleJSONInsert($insert_data)
	{
		return $this->genericQuery('insert', $insert_data);
	}
	
	/**
	 * Handle JSON requests with an 'update' action
	 *
	 * @access public
	 * @param Array The data to be updated.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handleJSONUpdate($update_data)
	{
		return $this->genericQuery('update', $update_data);
	}
	
	/**
	 * Handle page loads.
	 * 
	 * This method is called whenever a page is loaded.  It sets the 
	 * confirmation if any, if the action is 'edit' it ensures that an ID is 
	 * present, and fetches the data for the page if 
	 * <pre>$page_data['url_data'][2]</pre> is found.
	 * 
	 * @access public
	 * @param Array The page data.
	 * @return Void
	 */
	public function handlePageLoad($page_data)
	{		
		if (false == is_null(ASSession::valueForKey('error_code')))
		{
			global $error_messages;
			
			$this->PageRequest->setConfirmation
			(
				$error_messages[ASSession::valueForKey('error_code')]
			);
			
			ASSession::unsetValueForKey('error_code');
		}

		if ('edit' == $this->PageRequest->action())
		{
			if (false == isset($page_data['url_parts'][ASPageRequest::AS_PAGE_REQ_URL_PART_ID]))
			{
				throw new ASException("Missing id");
			}
			else
			{
				$this->PageRequest->setPageData($this->getDataAsArray());
			}
		}
	}

	/**
	 * Handle HTTP POST requests
	 * 
	 * This method is meant to be overridden so by default it does nothing.
	 *
	 * @access public
	 * @return Void
	 */
	public function handlePost()
	{
	}

	/**
	 * Handle HTTP POST requests with an 'insert' action
	 * 
	 * If the insert is successful then the user is redirected to the URL found
	 * in ASPageRequest::pathToAction().  Otherwise an error is set.
	 *
	 * @access public
	 * @param Array The data to be inserted.
	 * @return Boolean
	 * @see genericQuery()
	 * @see ASPageRequest::pathToAction()
	 */
	public function handlePostInsert($insert_data)
	{
		if ($this->genericQuery('insert', $insert_data))
		{
			header("Location: {$this->PageRequest->pathToAction()}list");
		}
		else
		{
			$this->PageRequest->setError
			(
				'There was an error inserting the data.'
			);
		}
	}

	/**
	 * Handle HTTP POST requests with an 'update' action
	 * 
	 * If the update is successful then the user is redirected to the URL found
	 * in ASPageRequest::pathToAction().  Otherwise an error is set.
	 *
	 * @access public
	 * @param Array $info_data
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handlePostUpdate($update_data)
	{
		if ($this->genericQuery('update', $update_data))
		{
			header("Location: {$this->PageRequest->pathToAction()}list");
		}
		else
		{
			$this->PageRequest->setError
			(
				'There was an error updating the data.'
			);
		}
	}

	/**
	 * Handle HTTP POST requests with a 'delete' action
	 *
	 * @access public
	 * @param Array The data to delete.
	 * @return Boolean
	 * @see genericQuery()
	 */
	public function handlePostDelete($delete_data)
	{
		return $this->genericQuery('delete', $delete_data);
	}

	/**
	 * Update the database based on an ASObserver event.
	 * 
	 * If the event is AS_PAGE_REQ_NOTICE_PAGE_LOADED then the handlePageLoad() 
	 * method is called.
	 * 
	 * If the event is AS_PAGE_REQ_NOTICE_GET_REQUESTED then one of handleGet(), 
	 * handleGetInsert(), handleGetUpdate(), or handleGetDelete() is called 
	 * depending on the value of $info['data']['form_action'].
	 * 
	 * If the event is AS_PAGE_REQ_NOTICE_POST_REQUESTED then one of 
	 * handlePost(), handlePostInsert(), handlePostUpdate(), or 
	 * handlePostDelete() is called depending on the value of 
	 * $info['data']['form_action'].
	 *
	 * @access public
	 * @param Array The data to be used in the query.
	 * @return Void
	 */
    public function updateFromObserver($info)
    {
    	// Initialize the form action to null if it wasn't present
    	// in the form.
		if (false == isset($info['data']['form_action']))
		{
			$info['data']['form_action'] = null;
		}

		// Check for a 'handlePageLoad()' method (hint: there should always be one present)
		// in the current Controller and pass $info to it.
		if ($info['code'] == ASPageRequest::AS_PAGE_REQ_NOTICE_PAGE_LOADED)
		{
			if (method_exists($this, 'handlePageLoad'))
			{
				$this->handlePageLoad($info['data']);
			}
		}
		
		// Handle various means of GET requests.  All data in $info was 
		// found in the URL query string.
		if ($info['code'] == ASPageRequest::AS_PAGE_REQ_NOTICE_GET_REQUESTED)
		{
			switch ($info['data']['form_action'])
			{
				// Create a new record in the db. 
				case 'insert':
				{
					if (method_exists($this, 'handleGetInsert'))
					{
						unset($info['data']['form_action']);
						$this->handleGetInsert($info['data']);
					}
					break;
				}
				// Update an existing record.
				case 'update':
				{
					if (method_exists($this, 'handleGetUpdate'))
					{
						unset($info['data']['form_action']);
						$this->handleGetUpdate($info['data']);
					}
					break;
				}
				// Delete an existing record. $info['data'] must contain the id.
				case 'delete':
				{
					if (method_exists($this, 'handleGetDelete'))
					{
						unset($info['data']['form_action']);
						$this->handleGetDelete($info['data']);
					}
					break;
				}
				// All other types of requests.
				default:
				{
					if (method_exists($this, 'handleGet'))
					{
						$this->handleGet($info['data']);
					}
				}
			}
		}
		// Handle various means of POST requests.
		else if ($info['code'] == ASPageRequest::AS_PAGE_REQ_NOTICE_POST_REQUESTED)
		{
			switch ($info['data']['form_action'])
			{
				// Create a new record in the db. 
				case 'insert':
				{
					if (method_exists($this, 'handlePostInsert'))
					{
						unset($info['data']['form_action']);
						$this->handlePostInsert($info['data']);
					}
					break;
				}
				// Update an existing record.
				case 'update':
				{
					if (method_exists($this, 'handlePostUpdate'))
					{
						unset($info['data']['form_action']);
						$this->handlePostUpdate($info['data']);
					}
					break;
				}
				// Delete an existing record.
				case 'delete':
				{
					if (method_exists($this, 'handlePostDelete'))
					{
						unset($info['data']['form_action']);
						$this->handlePostDelete($info['data']);
					}
					break;
				}
				// All other types of requests.
				default:
				{
					if (method_exists($this, 'handlePost'))
					{
						$this->handlePost($info['data']);
					}
				}
			}
		}
	}

	/**
	 * Get a value from ASDatabase by its associated key.
	 *
	 * @access public
	 * @param String $key
	 * @return Mixed
	 */
	public function valueForKey($key)
	{
		return $this->Model->getAttrib($key);
	}
}
?>
