/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview This class is automatically instanticated in the constructor
 * of ASApplication and stored in the AS_FORM global variable.  When
 * Instantiated it begins observing any forms on the page and traps
 * the submit event.  When that event is fired all the form fields
 * are processed into a JSON data format and sent to the server 
 * through XMLHttpRequest.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A class for processing form data.
 * @constructor
 * @returns An instance of ASForm.
 * @type Object
 */
function ASForm()
{
	/**
	 * A callback to be executed when the form has been submitted.
	 */
	this.form_submitted_callback = null;

	/**
	 * An array of hook functions that will be called before the
	 * data is submitted to the server.
	 *
	 * @type Array
	 */
	this.hooks = [];

	/**
	 * An array of form ID's which should not be observed.
	 * 
	 * @type Array
	 */
	this.ignore_forms = {};
	
	/**
	 * Whether the form data is ready to be sent
	 * to the server.
	 *
	 * @type Boolean
	 */
	this.ready_to_submit = true;

	this.observeForms();
}

// Extend ASObject
ASForm.prototype = new ASObject();

/**
 * A method that is called when the form has been submitted.
 *
 * @param The XMLHttpRequest data.
 * @param The XMLHttpRequest status.
 * @returns Void
 * @type Void
 */
ASForm.prototype._formSubmitted = function(data, status)
{
	if (null != AS_FORM.form_submitted_callback)
	{
		AS_FORM.form_submitted_callback(data, status);
	}
}

/**
 * Add an ID of a form to be ignored when observing is initialized.
 * 
 * @param form_id
 * @return Void
 */
ASForm.prototype.ignoreForm = function(form_id)
{
	this.ignore_forms[form_id] = true;
	
	var foo = this.ignore_forms;
}

/**
 * Observe any forms on the page.
 *
 * @see _prepare()
 * @returns Void.
 * @type Void
 */
ASForm.prototype.observeForms = function()
{
	// Get an array of all the forms on the page.
	var forms = document.getElementsByTagName('form');
	var x;

	// For every form in the array trap the submit event
	// sending it to _prepare().
	for (x=0; x<forms.length; x++)
	{
		if ('' == $(forms[x]).attr('id'))
		{
			continue;
		}
		
		$('#' + $(forms[x]).attr('id')).submit(this._prepare);
	}
}

/**
 * Prepare the JSON data object, check for required fields, and
 * submit the data to the server.
 *
 * @private
 * @param The event
 * @return Always returns false to prevent the browser from actually submitting the form.
 * @see _submit()
 * @type Boolean
 */
ASForm.prototype._prepare = function(event)
{
	if (true == AS_FORM.ignore_forms[event.target.id])
	{
		return true;
	}
	
	var frm			= $(event.target);
	var _data		= {};
	var x			= 0;
	var frm_action	= frm.attr('action');

	// Rewrite the form's action attribute to 'javascript: void(0)' so that we 
	// can control the location to which the data is submitted.
	// This way, if JavaScript is disabled then the form will still submit 
	// as the user expects.
	if (null == frm_action.match(/^javascript/))
	{
		$(frm[0]).attr('action', 'javascript: void(0)');
	}

	// Look through each input element and, excepting submit and reset elements
	// store the data about each one in the _data object.
	frm.find(':input').each(function(idx, el)
	{
		var element = $(el);

		if ('submit' == element.attr('type') || 'reset' == element.attr('type'))
		{
			return;
		}
		
		if ('radio' == element.attr('type'))
		{
			if (true !== element[0].checked)
			{
				return;
			}
		}
		
		if ('checkbox' == element.attr('type'))
		{
			if (true !== element[0].checked)
			{
				return;
			}
		}
		
		_data[element.attr('name')] = element.attr('value');
	});

	// Loop through the hooks array and call each one, passing frm as the param.
	for (x in AS_FORM.hooks)
	{
		if (false == AS_FORM.hooks[x](frm))
		{
			return false;
		}
	}

	// If all is well with the form then send the data to the server.
	if (AS_FORM.ready_to_submit)
	{
		if (undefined == _data['page'])
		{
			_data['page'] = page();
		}
		
		AS_FORM._submit
		(
			frm.attr('name') || frm.attr('id')
		  , encodeURIComponent(JSON.stringify(_data, null, 4))
		);
	}

	return false;
}

/**
 *
 */
ASForm.prototype.registerFormSubmittedCallback = function(form_submitted_callback)
{
	this.form_submitted_callback = form_submitted_callback;
}

/**
 * Add a function to the hooks property.  When _prepare is called
 * it loops through each hook in the array and calls them, passing 
 * the form as the parameter.  If the hook function returns false
 * the form data isn't sent to the server.
 *
 * NOTE: This method should only be called inside the onload function.
 * i.e. $(function(){ AS_FORM.registerHook(sampleHook) });
 *
 * @see _prepare()
 * @returns Void.
 * @type Void
 */
ASForm.prototype.registerHook = function(hook)
{
	AS_FORM.hooks.push(hook);
}

/**
 * Submit the JSON encoded data to the server.
 *
 * The server url is always: http://domain.tld/[dirX/]alchitect/server.php.  
 * For example, if you access Alchitect from the URL 'http://example.com/alchitect/index.html'
 * then the data will always be submitted to 'http://example.com/alchitect/server.php'.
 *
 * @private
 * @param The name of the form.  This is used on the server to determine where to route the data.
 * @param The JSON encoded data.
 * @returns Void.
 * @type Void
 */
ASForm.prototype._submit = function(form_name, data)
{
	// Submit the data to the server through Ajax
	$.post(
		URLPath() + '/core/server.php'
	  ,  '&form_name=' + form_name + '&json=' + data
	  , this._formSubmitted
	  , "json"
	);
}

/**
 * 
 * @param frm
 * @return
 */
ASForm.prototype.checkRequiredHook = function(frm)
{
	var ready_to_submit = true;
	var required = frm.find('input[required="true"]');

	// Loop through all input elements that have a 'required="true"' attribute 
	// and make sure the value of each element isn't empty.  If so then apply 
	// the 'error_input' class to it and don't submit the data.
	required.each(
		function(idx, el)
		{
			var element = $(el);

			if ('' == element.val())
			{
				element.parent().addClass('error_input');
				ready_to_submit=false;

				if (0 == idx)
				{
					element.focus();
				}
			}
			else
			{
				element.parent().removeClass('error_input');
				ready_to_submit=true;
			}
		}
	);

	return ready_to_submit;
}
